package javax.swing;

import java.awt.event.FocusListener;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Cursor;
import java.awt.Color;
import javax.accessibility.AccessibleValue;
import javax.accessibility.AccessibleText;
import java.util.Locale;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleAction;
import javax.accessibility.AccessibleComponent;
import javax.accessibility.AccessibleRole;
import java.beans.PropertyChangeEvent;
import javax.accessibility.AccessibleSelection;
import java.beans.PropertyChangeListener;
import javax.swing.tree.RowMapper;
import java.awt.event.ActionEvent;
import javax.accessibility.AccessibleContext;
import javax.swing.event.TreeModelEvent;
import java.awt.Container;
import java.awt.Dimension;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.io.ObjectOutputStream;
import javax.swing.text.Position;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.ExpandVetoException;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.event.TreeSelectionListener;
import java.util.Collections;
import java.util.Enumeration;
import java.awt.Component;
import sun.awt.AWTAccessor;
import java.awt.event.MouseEvent;
import java.awt.Rectangle;
import sun.swing.SwingUtilities2;
import java.awt.Point;
import java.awt.HeadlessException;
import java.awt.GraphicsEnvironment;
import javax.accessibility.AccessibleState;
import javax.swing.plaf.TreeUI;
import java.beans.ConstructorProperties;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeNode;
import java.util.Vector;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeExpansionListener;
import java.util.Stack;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreePath;
import java.util.Hashtable;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreeModel;
import javax.accessibility.Accessible;

public class JTree extends JComponent implements Scrollable, Accessible
{
    private static final String uiClassID = "TreeUI";
    protected transient TreeModel treeModel;
    protected transient TreeSelectionModel selectionModel;
    protected boolean rootVisible;
    protected transient TreeCellRenderer cellRenderer;
    protected int rowHeight;
    private boolean rowHeightSet;
    private transient Hashtable<TreePath, Boolean> expandedState;
    protected boolean showsRootHandles;
    private boolean showsRootHandlesSet;
    protected transient JTree.TreeSelectionRedirector selectionRedirector;
    protected transient TreeCellEditor cellEditor;
    protected boolean editable;
    protected boolean largeModel;
    protected int visibleRowCount;
    protected boolean invokesStopCellEditing;
    protected boolean scrollsOnExpand;
    private boolean scrollsOnExpandSet;
    protected int toggleClickCount;
    protected transient TreeModelListener treeModelListener;
    private transient Stack<Stack<TreePath>> expandedStack;
    private TreePath leadPath;
    private TreePath anchorPath;
    private boolean expandsSelectedPaths;
    private boolean settingUI;
    private boolean dragEnabled;
    private DropMode dropMode;
    private transient JTree.DropLocation dropLocation;
    private int expandRow;
    private JTree.TreeTimer dropTimer;
    private transient TreeExpansionListener uiTreeExpansionListener;
    private static int TEMP_STACK_SIZE;
    public static final String CELL_RENDERER_PROPERTY = "cellRenderer";
    public static final String TREE_MODEL_PROPERTY = "model";
    public static final String ROOT_VISIBLE_PROPERTY = "rootVisible";
    public static final String SHOWS_ROOT_HANDLES_PROPERTY = "showsRootHandles";
    public static final String ROW_HEIGHT_PROPERTY = "rowHeight";
    public static final String CELL_EDITOR_PROPERTY = "cellEditor";
    public static final String EDITABLE_PROPERTY = "editable";
    public static final String LARGE_MODEL_PROPERTY = "largeModel";
    public static final String SELECTION_MODEL_PROPERTY = "selectionModel";
    public static final String VISIBLE_ROW_COUNT_PROPERTY = "visibleRowCount";
    public static final String INVOKES_STOP_CELL_EDITING_PROPERTY = "invokesStopCellEditing";
    public static final String SCROLLS_ON_EXPAND_PROPERTY = "scrollsOnExpand";
    public static final String TOGGLE_CLICK_COUNT_PROPERTY = "toggleClickCount";
    public static final String LEAD_SELECTION_PATH_PROPERTY = "leadSelectionPath";
    public static final String ANCHOR_SELECTION_PATH_PROPERTY = "anchorSelectionPath";
    public static final String EXPANDS_SELECTED_PATHS_PROPERTY = "expandsSelectedPaths";
    
    protected static TreeModel getDefaultTreeModel() {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode("JTree");
        DefaultMutableTreeNode newChild = new DefaultMutableTreeNode("colors");
        root.add(newChild);
        newChild.add(new DefaultMutableTreeNode("blue"));
        newChild.add(new DefaultMutableTreeNode("violet"));
        newChild.add(new DefaultMutableTreeNode("red"));
        newChild.add(new DefaultMutableTreeNode("yellow"));
        DefaultMutableTreeNode newChild2 = new DefaultMutableTreeNode("sports");
        root.add(newChild2);
        newChild2.add(new DefaultMutableTreeNode("basketball"));
        newChild2.add(new DefaultMutableTreeNode("soccer"));
        newChild2.add(new DefaultMutableTreeNode("football"));
        newChild2.add(new DefaultMutableTreeNode("hockey"));
        DefaultMutableTreeNode newChild3 = new DefaultMutableTreeNode("food");
        root.add(newChild3);
        newChild3.add(new DefaultMutableTreeNode("hot dogs"));
        newChild3.add(new DefaultMutableTreeNode("pizza"));
        newChild3.add(new DefaultMutableTreeNode("ravioli"));
        newChild3.add(new DefaultMutableTreeNode("bananas"));
        return new DefaultTreeModel(root);
    }
    
    protected static TreeModel createTreeModel(Object o) {
        DefaultMutableTreeNode root;
        if (o instanceof Object[] || o instanceof Hashtable || o instanceof Vector) {
            root = new DefaultMutableTreeNode("root");
            JTree.DynamicUtilTreeNode.createChildren(root, o);
        }
        else {
            root = new DynamicUtilTreeNode("root", o);
        }
        return new DefaultTreeModel(root, false);
    }
    
    public JTree() {
        this(getDefaultTreeModel());
    }
    
    public JTree(Object[] array) {
        this(createTreeModel(array));
        this.setRootVisible(false);
        this.setShowsRootHandles(true);
        this.expandRoot();
    }
    
    public JTree(Vector<?> vector) {
        this(createTreeModel(vector));
        this.setRootVisible(false);
        this.setShowsRootHandles(true);
        this.expandRoot();
    }
    
    public JTree(Hashtable<?, ?> hashtable) {
        this(createTreeModel(hashtable));
        this.setRootVisible(false);
        this.setShowsRootHandles(true);
        this.expandRoot();
    }
    
    public JTree(TreeNode treeNode) {
        this(treeNode, false);
    }
    
    public JTree(TreeNode root, boolean asksAllowsChildren) {
        this(new DefaultTreeModel(root, asksAllowsChildren));
    }
    
    @ConstructorProperties({ "model" })
    public JTree(TreeModel model) {
        this.rowHeightSet = false;
        this.showsRootHandlesSet = false;
        this.scrollsOnExpandSet = false;
        this.dropMode = DropMode.USE_SELECTION;
        this.expandRow = -1;
        this.expandedStack = new Stack();
        this.toggleClickCount = 2;
        this.expandedState = new Hashtable();
        this.setLayout(null);
        this.rowHeight = 16;
        this.visibleRowCount = 20;
        this.rootVisible = true;
        this.selectionModel = new DefaultTreeSelectionModel();
        this.cellRenderer = null;
        this.setOpaque(this.scrollsOnExpand = true);
        this.expandsSelectedPaths = true;
        this.updateUI();
        this.setModel(model);
    }
    
    public TreeUI getUI() {
        return (TreeUI)this.ui;
    }
    
    public void setUI(TreeUI ui) {
        if (this.ui != ui) {
            this.settingUI = true;
            this.uiTreeExpansionListener = null;
            try {
                super.setUI(ui);
            }
            finally {
                this.settingUI = false;
            }
        }
    }
    
    @Override
    public void updateUI() {
        this.setUI((TreeUI)UIManager.getUI(this));
        SwingUtilities.updateRendererOrEditorUI(this.getCellRenderer());
        SwingUtilities.updateRendererOrEditorUI(this.getCellEditor());
    }
    
    @Override
    public String getUIClassID() {
        return "TreeUI";
    }
    
    public TreeCellRenderer getCellRenderer() {
        return this.cellRenderer;
    }
    
    public void setCellRenderer(TreeCellRenderer cellRenderer) {
        this.firePropertyChange("cellRenderer", this.cellRenderer, this.cellRenderer = cellRenderer);
        this.invalidate();
    }
    
    public void setEditable(boolean editable) {
        int editable2 = this.editable;
        this.firePropertyChange("editable", (boolean)editable2, this.editable = editable);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleState", editable2 ? AccessibleState.EDITABLE : null, editable ? AccessibleState.EDITABLE : null);
        }
    }
    
    public boolean isEditable() {
        return this.editable;
    }
    
    public void setCellEditor(TreeCellEditor cellEditor) {
        this.firePropertyChange("cellEditor", this.cellEditor, this.cellEditor = cellEditor);
        this.invalidate();
    }
    
    public TreeCellEditor getCellEditor() {
        return this.cellEditor;
    }
    
    public TreeModel getModel() {
        return this.treeModel;
    }
    
    public void setModel(TreeModel treeModel) {
        this.clearSelection();
        TreeModel treeModel2 = this.treeModel;
        if (this.treeModel != null && this.treeModelListener != null) {
            this.treeModel.removeTreeModelListener(this.treeModelListener);
        }
        if (this.accessibleContext != null) {
            if (this.treeModel != null) {
                this.treeModel.removeTreeModelListener((TreeModelListener)this.accessibleContext);
            }
            if (treeModel != null) {
                treeModel.addTreeModelListener((TreeModelListener)this.accessibleContext);
            }
        }
        this.treeModel = treeModel;
        this.clearToggledPaths();
        if (this.treeModel != null) {
            if (this.treeModelListener == null) {
                this.treeModelListener = this.createTreeModelListener();
            }
            if (this.treeModelListener != null) {
                this.treeModel.addTreeModelListener(this.treeModelListener);
            }
            Object root = this.treeModel.getRoot();
            if (root != null && !this.treeModel.isLeaf(root)) {
                this.expandedState.put(new TreePath(root), Boolean.TRUE);
            }
        }
        this.firePropertyChange("model", treeModel2, this.treeModel);
        this.invalidate();
    }
    
    public boolean isRootVisible() {
        return this.rootVisible;
    }
    
    public void setRootVisible(boolean rootVisible) {
        this.firePropertyChange("rootVisible", this.rootVisible, this.rootVisible = rootVisible);
        if (this.accessibleContext != null) {
            ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
        }
    }
    
    public void setShowsRootHandles(boolean showsRootHandles) {
        int showsRootHandles2 = this.showsRootHandles;
        this.getModel();
        this.showsRootHandles = showsRootHandles;
        this.showsRootHandlesSet = true;
        this.firePropertyChange("showsRootHandles", (boolean)showsRootHandles2, this.showsRootHandles);
        if (this.accessibleContext != null) {
            ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
        }
        this.invalidate();
    }
    
    public boolean getShowsRootHandles() {
        return this.showsRootHandles;
    }
    
    public void setRowHeight(int rowHeight) {
        int rowHeight2 = this.rowHeight;
        this.rowHeight = rowHeight;
        this.rowHeightSet = true;
        this.firePropertyChange("rowHeight", rowHeight2, this.rowHeight);
        this.invalidate();
    }
    
    public int getRowHeight() {
        return this.rowHeight;
    }
    
    public boolean isFixedRowHeight() {
        return this.rowHeight > 0;
    }
    
    public void setLargeModel(boolean largeModel) {
        this.firePropertyChange("largeModel", this.largeModel, this.largeModel = largeModel);
    }
    
    public boolean isLargeModel() {
        return this.largeModel;
    }
    
    public void setInvokesStopCellEditing(boolean invokesStopCellEditing) {
        this.firePropertyChange("invokesStopCellEditing", this.invokesStopCellEditing, this.invokesStopCellEditing = invokesStopCellEditing);
    }
    
    public boolean getInvokesStopCellEditing() {
        return this.invokesStopCellEditing;
    }
    
    public void setScrollsOnExpand(boolean scrollsOnExpand) {
        int scrollsOnExpand2 = this.scrollsOnExpand;
        this.scrollsOnExpand = scrollsOnExpand;
        this.scrollsOnExpandSet = true;
        this.firePropertyChange("scrollsOnExpand", (boolean)scrollsOnExpand2, scrollsOnExpand);
    }
    
    public boolean getScrollsOnExpand() {
        return this.scrollsOnExpand;
    }
    
    public void setToggleClickCount(int toggleClickCount) {
        this.firePropertyChange("toggleClickCount", this.toggleClickCount, this.toggleClickCount = toggleClickCount);
    }
    
    public int getToggleClickCount() {
        return this.toggleClickCount;
    }
    
    public void setExpandsSelectedPaths(boolean expandsSelectedPaths) {
        this.firePropertyChange("expandsSelectedPaths", this.expandsSelectedPaths, this.expandsSelectedPaths = expandsSelectedPaths);
    }
    
    public boolean getExpandsSelectedPaths() {
        return this.expandsSelectedPaths;
    }
    
    public void setDragEnabled(boolean dragEnabled) {
        if (dragEnabled && GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        this.dragEnabled = dragEnabled;
        return;
    }
    
    public boolean getDragEnabled() {
        return this.dragEnabled;
    }
    
    public final void setDropMode(DropMode dropMode) {
        if (dropMode != null) {
            switch (dropMode) {
                case USE_SELECTION:
                case ON:
                case INSERT:
                case ON_OR_INSERT: {
                    this.dropMode = dropMode;
                    return;
                }
                default: {
                    break;
                }
            }
        }
        throw new IllegalArgumentException(dropMode + ": Unsupported drop mode for tree");
    }
    
    public final DropMode getDropMode() {
        return this.dropMode;
    }
    
    @Override
    JTree.DropLocation dropLocationForPoint(Point p) {
        Object o = null;
        int closestRowForLocation = this.getClosestRowForLocation(p.x, p.y);
        Rectangle rowBounds = this.getRowBounds(closestRowForLocation);
        TreeModel model = this.getModel();
        Object lastPathComponent = (model == null) ? null : model.getRoot();
        TreePath treePath = (lastPathComponent == null) ? null : new TreePath(lastPathComponent);
        int n = closestRowForLocation == -1 || p.y < rowBounds.y || p.y >= rowBounds.y + rowBounds.height;
        switch (this.dropMode) {
            case USE_SELECTION:
            case ON: {
                if (!(n == 0)) {
                    o = new JTree.DropLocation(p, null, -1);
                    break;
                }
                o = new JTree.DropLocation(p, this.getPathForRow(closestRowForLocation), -1);
                break;
            }
            case INSERT:
            case ON_OR_INSERT: {
                if (closestRowForLocation == -1) {
                    if (lastPathComponent != null && !model.isLeaf(lastPathComponent) && this.isExpanded(treePath)) {
                        o = new JTree.DropLocation(p, treePath, 0);
                        break;
                    }
                    o = new JTree.DropLocation(p, null, -1);
                    break;
                }
                else {
                    int three = this.dropMode == DropMode.ON_OR_INSERT || !model.isLeaf(this.getPathForRow(closestRowForLocation).getLastPathComponent());
                    SwingUtilities2.Section liesInVertical = SwingUtilities2.liesInVertical(rowBounds, p, (boolean)three);
                    TreePath treePath2;
                    TreePath treePath3;
                    if (liesInVertical == SwingUtilities2.Section.LEADING) {
                        treePath2 = this.getPathForRow(closestRowForLocation);
                        treePath3 = treePath2.getParentPath();
                    }
                    else if (liesInVertical == SwingUtilities2.Section.TRAILING) {
                        int n2 = closestRowForLocation + 1;
                        if (n2 >= this.getRowCount()) {
                            if (model.isLeaf(lastPathComponent) || !this.isExpanded(treePath)) {
                                o = new JTree.DropLocation(p, null, -1);
                                break;
                            }
                            o = new JTree.DropLocation(p, treePath, model.getChildCount(lastPathComponent));
                            break;
                        }
                        else {
                            treePath2 = this.getPathForRow(n2);
                            treePath3 = treePath2.getParentPath();
                        }
                    }
                    else {
                        assert !(three == 0);
                        o = new JTree.DropLocation(p, this.getPathForRow(closestRowForLocation), -1);
                        break;
                    }
                    if (treePath3 != null) {
                        o = new JTree.DropLocation(p, treePath3, model.getIndexOfChild(treePath3.getLastPathComponent(), treePath2.getLastPathComponent()));
                        break;
                    }
                    if (three || !model.isLeaf(lastPathComponent)) {
                        o = new JTree.DropLocation(p, treePath, -1);
                        break;
                    }
                    o = new JTree.DropLocation(p, null, -1);
                    break;
                }
            }
            default: {
                assert false : "Unexpected drop mode";
                break;
            }
        }
        if (n || closestRowForLocation != this.expandRow) {
            this.cancelDropTimer();
        }
        if (n == 0 && closestRowForLocation != this.expandRow && this.isCollapsed(closestRowForLocation)) {
            this.expandRow = closestRowForLocation;
            this.startDropTimer();
            return o;
        }
        return o;
    }
    
    @Override
    Object setDropLocation(TransferHandler.DropLocation dropLocation, Object o, boolean b) {
        Object o2 = null;
        JTree.DropLocation dropLocation2 = (JTree.DropLocation)dropLocation;
        if (this.dropMode == DropMode.USE_SELECTION) {
            if (dropLocation2 == null) {
                if (!b && o != null) {
                    this.setSelectionPaths(((TreePath[][])(TreePath[][])o)[0]);
                    this.setAnchorSelectionPath(((TreePath[][])(TreePath[][])o)[1][0]);
                    this.setLeadSelectionPath(((TreePath[][])(TreePath[][])o)[1][1]);
                }
            }
            else {
                if (this.dropLocation == null) {
                    TreePath[] selectionPaths = this.getSelectionPaths();
                    if (selectionPaths == null) {
                        selectionPaths = new TreePath[0];
                    }
                    o2 = new TreePath[][] { selectionPaths, { this.getAnchorSelectionPath(), this.getLeadSelectionPath() } };
                }
                else {
                    o2 = o;
                }
                this.setSelectionPath(dropLocation2.getPath());
            }
        }
        this.firePropertyChange("dropLocation", this.dropLocation, this.dropLocation = dropLocation2);
        return o2;
    }
    
    @Override
    void dndDone() {
        this.cancelDropTimer();
        this.dropTimer = null;
    }
    
    public final JTree.DropLocation getDropLocation() {
        return this.dropLocation;
    }
    
    private void startDropTimer() {
        if (this.dropTimer == null) {
            this.dropTimer = new JTree.TreeTimer();
        }
        this.dropTimer.start();
    }
    
    private void cancelDropTimer() {
        if (this.dropTimer != null && this.dropTimer.isRunning()) {
            this.expandRow = -1;
            this.dropTimer.stop();
        }
    }
    
    public boolean isPathEditable(TreePath treePath) {
        return this.isEditable();
    }
    
    @Override
    public String getToolTipText(MouseEvent mouseEvent) {
        String s = null;
        if (mouseEvent != null) {
            Point point = mouseEvent.getPoint();
            int rowForLocation = this.getRowForLocation(point.x, point.y);
            TreeCellRenderer cellRenderer = this.getCellRenderer();
            if (rowForLocation != -1 && cellRenderer != null) {
                TreePath pathForRow = this.getPathForRow(rowForLocation);
                Object lastPathComponent = pathForRow.getLastPathComponent();
                Component treeCellRendererComponent = cellRenderer.getTreeCellRendererComponent(this, lastPathComponent, this.isRowSelected(rowForLocation), this.isExpanded(rowForLocation), this.getModel().isLeaf(lastPathComponent), rowForLocation, true);
                if ((treeCellRendererComponent instanceof JComponent)) {
                    Rectangle pathBounds = this.getPathBounds(pathForRow);
                    point.translate(-pathBounds.x, -pathBounds.y);
                    MouseEvent mouseEvent2 = new MouseEvent(treeCellRendererComponent, mouseEvent.getID(), mouseEvent.getWhen(), mouseEvent.getModifiers(), point.x, point.y, mouseEvent.getXOnScreen(), mouseEvent.getYOnScreen(), mouseEvent.getClickCount(), mouseEvent.isPopupTrigger(), 0);
                    AWTAccessor.MouseEventAccessor mouseEventAccessor = AWTAccessor.getMouseEventAccessor();
                    mouseEventAccessor.setCausedByTouchEvent(mouseEvent2, mouseEventAccessor.isCausedByTouchEvent(mouseEvent));
                    s = ((JComponent)treeCellRendererComponent).getToolTipText(mouseEvent2);
                }
            }
        }
        if (s == null) {
            s = super.getToolTipText();
            return s;
        }
        return s;
    }
    
    public String convertValueToText(Object o, boolean b, boolean b2, boolean b3, int n, boolean b4) {
        if (o != null) {
            String string = o.toString();
            if (string != null) {
                return string;
            }
        }
        return "";
    }
    
    public int getRowCount() {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getRowCount(this);
        }
        return 0;
    }
    
    public void setSelectionPath(TreePath selectionPath) {
        this.getSelectionModel().setSelectionPath(selectionPath);
    }
    
    public void setSelectionPaths(TreePath[] selectionPaths) {
        this.getSelectionModel().setSelectionPaths(selectionPaths);
    }
    
    public void setLeadSelectionPath(TreePath leadPath) {
        TreePath leadPath2 = this.leadPath;
        this.firePropertyChange("leadSelectionPath", leadPath2, this.leadPath = leadPath);
        if (this.accessibleContext != null) {
            ((JTree.AccessibleJTree)this.accessibleContext).fireActiveDescendantPropertyChange(leadPath2, leadPath);
        }
    }
    
    public void setAnchorSelectionPath(TreePath anchorPath) {
        this.firePropertyChange("anchorSelectionPath", this.anchorPath, this.anchorPath = anchorPath);
    }
    
    public void setSelectionRow(int n) {
        this.setSelectionRows(new int[] { n });
    }
    
    public void setSelectionRows(int[] array) {
        TreeUI ui = this.getUI();
        if (ui != null && array != null) {
            int length = array.length;
            TreePath[] selectionPaths = new TreePath[length];
            for (int i = 0; i < length; ++i) {
                selectionPaths[i] = ui.getPathForRow(this, array[i]);
            }
            this.setSelectionPaths(selectionPaths);
        }
    }
    
    public void addSelectionPath(TreePath treePath) {
        this.getSelectionModel().addSelectionPath(treePath);
    }
    
    public void addSelectionPaths(TreePath[] array) {
        this.getSelectionModel().addSelectionPaths(array);
    }
    
    public void addSelectionRow(int n) {
        this.addSelectionRows(new int[] { n });
    }
    
    public void addSelectionRows(int[] array) {
        TreeUI ui = this.getUI();
        if (ui != null && array != null) {
            int length = array.length;
            TreePath[] array2 = new TreePath[length];
            for (int i = 0; i < length; ++i) {
                array2[i] = ui.getPathForRow(this, array[i]);
            }
            this.addSelectionPaths(array2);
        }
    }
    
    public Object getLastSelectedPathComponent() {
        TreePath selectionPath = this.getSelectionModel().getSelectionPath();
        if (selectionPath != null) {
            return selectionPath.getLastPathComponent();
        }
        return null;
    }
    
    public TreePath getLeadSelectionPath() {
        return this.leadPath;
    }
    
    public TreePath getAnchorSelectionPath() {
        return this.anchorPath;
    }
    
    public TreePath getSelectionPath() {
        return this.getSelectionModel().getSelectionPath();
    }
    
    public TreePath[] getSelectionPaths() {
        TreePath[] selectionPaths = this.getSelectionModel().getSelectionPaths();
        return (selectionPaths != null && selectionPaths.length > 0) ? selectionPaths : null;
    }
    
    public int[] getSelectionRows() {
        return this.getSelectionModel().getSelectionRows();
    }
    
    public int getSelectionCount() {
        return this.selectionModel.getSelectionCount();
    }
    
    public int getMinSelectionRow() {
        return this.getSelectionModel().getMinSelectionRow();
    }
    
    public int getMaxSelectionRow() {
        return this.getSelectionModel().getMaxSelectionRow();
    }
    
    public int getLeadSelectionRow() {
        TreePath leadSelectionPath = this.getLeadSelectionPath();
        if (leadSelectionPath != null) {
            return this.getRowForPath(leadSelectionPath);
        }
        return -1;
    }
    
    public boolean isPathSelected(TreePath treePath) {
        return this.getSelectionModel().isPathSelected(treePath);
    }
    
    public boolean isRowSelected(int n) {
        return this.getSelectionModel().isRowSelected(n);
    }
    
    public Enumeration<TreePath> getExpandedDescendants(TreePath treePath) {
        if (!this.isExpanded(treePath)) {
            return null;
        }
        Enumeration<TreePath> keys = this.expandedState.keys();
        Object o = null;
        if (keys != null) {
            while (keys.hasMoreElements()) {
                TreePath obj = (TreePath)keys.nextElement();
                Object value = this.expandedState.get(obj);
                if (obj == treePath || value == null || !(boolean)value || !treePath.isDescendant(obj) || !this.isVisible(obj)) {
                    continue;
                }
                if (o == null) {
                    o = new Vector();
                }
                o.addElement(obj);
            }
        }
        if (o == null) {
            return Collections.enumeration(Collections.emptySet());
        }
        return o.elements();
    }
    
    public boolean hasBeenExpanded(TreePath key) {
        return key != null && this.expandedState.get(key) != null;
    }
    
    public boolean isExpanded(TreePath parentPath) {
        if (parentPath == null) {
            return false;
        }
        do {
            Object value = this.expandedState.get(parentPath);
            if (value == null || !(boolean)value) {
                return false;
            }
            continue;
        } while ((parentPath = parentPath.getParentPath()) != null);
        return true;
    }
    
    public boolean isExpanded(int n) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            TreePath pathForRow = ui.getPathForRow(this, n);
            if (pathForRow != null) {
                Boolean b = (Boolean)this.expandedState.get(pathForRow);
                return b != null && (boolean)b;
            }
        }
        return false;
    }
    
    public boolean isCollapsed(TreePath treePath) {
        return !this.isExpanded(treePath);
    }
    
    public boolean isCollapsed(int n) {
        return !this.isExpanded(n);
    }
    
    public void makeVisible(TreePath treePath) {
        if (treePath != null) {
            TreePath parentPath = treePath.getParentPath();
            if (parentPath != null) {
                this.expandPath(parentPath);
            }
        }
    }
    
    public boolean isVisible(TreePath treePath) {
        if (treePath != null) {
            TreePath parentPath = treePath.getParentPath();
            return parentPath == null || this.isExpanded(parentPath);
        }
        return false;
    }
    
    public Rectangle getPathBounds(TreePath treePath) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getPathBounds(this, treePath);
        }
        return null;
    }
    
    public Rectangle getRowBounds(int n) {
        return this.getPathBounds(this.getPathForRow(n));
    }
    
    public void scrollPathToVisible(TreePath treePath) {
        if (treePath != null) {
            this.makeVisible(treePath);
            Rectangle pathBounds = this.getPathBounds(treePath);
            if (pathBounds != null) {
                this.scrollRectToVisible(pathBounds);
                if (this.accessibleContext != null) {
                    ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
                }
            }
        }
    }
    
    public void scrollRowToVisible(int n) {
        this.scrollPathToVisible(this.getPathForRow(n));
    }
    
    public TreePath getPathForRow(int n) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getPathForRow(this, n);
        }
        return null;
    }
    
    public int getRowForPath(TreePath treePath) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getRowForPath(this, treePath);
        }
        return -1;
    }
    
    public void expandPath(TreePath treePath) {
        TreeModel model = this.getModel();
        if (treePath != null && model != null && !model.isLeaf(treePath.getLastPathComponent())) {
            this.setExpandedState(treePath, true);
        }
    }
    
    public void expandRow(int n) {
        this.expandPath(this.getPathForRow(n));
    }
    
    public void collapsePath(TreePath treePath) {
        this.setExpandedState(treePath, false);
    }
    
    public void collapseRow(int n) {
        this.collapsePath(this.getPathForRow(n));
    }
    
    public TreePath getPathForLocation(int n, int n2) {
        TreePath closestPathForLocation = this.getClosestPathForLocation(n, n2);
        if (closestPathForLocation != null) {
            Rectangle pathBounds = this.getPathBounds(closestPathForLocation);
            if (pathBounds != null && n >= pathBounds.x && n < pathBounds.x + pathBounds.width && n2 >= pathBounds.y && n2 < pathBounds.y + pathBounds.height) {
                return closestPathForLocation;
            }
        }
        return null;
    }
    
    public int getRowForLocation(int n, int n2) {
        return this.getRowForPath(this.getPathForLocation(n, n2));
    }
    
    public TreePath getClosestPathForLocation(int n, int n2) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getClosestPathForLocation(this, n, n2);
        }
        return null;
    }
    
    public int getClosestRowForLocation(int n, int n2) {
        return this.getRowForPath(this.getClosestPathForLocation(n, n2));
    }
    
    public boolean isEditing() {
        TreeUI ui = this.getUI();
        return ui != null && ui.isEditing(this);
    }
    
    public boolean stopEditing() {
        TreeUI ui = this.getUI();
        return ui != null && ui.stopEditing(this);
    }
    
    public void cancelEditing() {
        TreeUI ui = this.getUI();
        if (ui != null) {
            ui.cancelEditing(this);
        }
    }
    
    public void startEditingAtPath(TreePath treePath) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            ui.startEditingAtPath(this, treePath);
        }
    }
    
    public TreePath getEditingPath() {
        TreeUI ui = this.getUI();
        if (ui != null) {
            return ui.getEditingPath(this);
        }
        return null;
    }
    
    public void setSelectionModel(TreeSelectionModel sharedInstance) {
        if (sharedInstance == null) {
            sharedInstance = JTree.EmptySelectionModel.sharedInstance();
        }
        TreeSelectionModel selectionModel = this.selectionModel;
        if (this.selectionModel != null && this.selectionRedirector != null) {
            this.selectionModel.removeTreeSelectionListener(this.selectionRedirector);
        }
        if (this.accessibleContext != null) {
            this.selectionModel.removeTreeSelectionListener((TreeSelectionListener)this.accessibleContext);
            sharedInstance.addTreeSelectionListener((TreeSelectionListener)this.accessibleContext);
        }
        this.selectionModel = sharedInstance;
        if (this.selectionRedirector != null) {
            this.selectionModel.addTreeSelectionListener(this.selectionRedirector);
        }
        this.firePropertyChange("selectionModel", selectionModel, this.selectionModel);
        if (this.accessibleContext != null) {
            this.accessibleContext.firePropertyChange("AccessibleSelection", Boolean.valueOf(false), Boolean.valueOf(true));
        }
    }
    
    public TreeSelectionModel getSelectionModel() {
        return this.selectionModel;
    }
    
    protected TreePath[] getPathBetweenRows(int min, int min2) {
        TreeUI ui = this.getUI();
        if (ui != null) {
            int rowCount = this.getRowCount();
            if (rowCount > 0 && (min >= 0 || min2 >= 0) && (min < rowCount || min2 < rowCount)) {
                min = Math.min(rowCount - 1, Math.max(min, 0));
                min2 = Math.min(rowCount - 1, Math.max(min2, 0));
                int i = Math.min(min, min2);
                int max = Math.max(min, min2);
                TreePath[] array = new TreePath[max - i + 1];
                for (; i <= max; ++i) {
                    array[i - i] = ui.getPathForRow(this, i);
                }
                return array;
            }
        }
        return new TreePath[0];
    }
    
    public void setSelectionInterval(int n, int n2) {
        this.getSelectionModel().setSelectionPaths(this.getPathBetweenRows(n, n2));
    }
    
    public void addSelectionInterval(int n, int n2) {
        TreePath[] pathBetweenRows = this.getPathBetweenRows(n, n2);
        if (pathBetweenRows != null && pathBetweenRows.length > 0) {
            this.getSelectionModel().addSelectionPaths(pathBetweenRows);
        }
    }
    
    public void removeSelectionInterval(int n, int n2) {
        TreePath[] pathBetweenRows = this.getPathBetweenRows(n, n2);
        if (pathBetweenRows != null && pathBetweenRows.length > 0) {
            this.getSelectionModel().removeSelectionPaths(pathBetweenRows);
        }
    }
    
    public void removeSelectionPath(TreePath treePath) {
        this.getSelectionModel().removeSelectionPath(treePath);
    }
    
    public void removeSelectionPaths(TreePath[] array) {
        this.getSelectionModel().removeSelectionPaths(array);
    }
    
    public void removeSelectionRow(int n) {
        this.removeSelectionRows(new int[] { n });
    }
    
    public void removeSelectionRows(int[] array) {
        TreeUI ui = this.getUI();
        if (ui != null && array != null) {
            int length = array.length;
            TreePath[] array2 = new TreePath[length];
            for (int i = 0; i < length; ++i) {
                array2[i] = ui.getPathForRow(this, array[i]);
            }
            this.removeSelectionPaths(array2);
        }
    }
    
    public void clearSelection() {
        this.getSelectionModel().clearSelection();
    }
    
    public boolean isSelectionEmpty() {
        return this.getSelectionModel().isSelectionEmpty();
    }
    
    public void addTreeExpansionListener(TreeExpansionListener treeExpansionListener) {
        if (this.settingUI) {
            this.uiTreeExpansionListener = treeExpansionListener;
        }
        this.listenerList.add(TreeExpansionListener.class, treeExpansionListener);
    }
    
    public void removeTreeExpansionListener(TreeExpansionListener l) {
        this.listenerList.remove(TreeExpansionListener.class, l);
        if (this.uiTreeExpansionListener == l) {
            this.uiTreeExpansionListener = null;
        }
    }
    
    public TreeExpansionListener[] getTreeExpansionListeners() {
        return (TreeExpansionListener[])this.listenerList.getListeners(TreeExpansionListener.class);
    }
    
    public void addTreeWillExpandListener(TreeWillExpandListener l) {
        this.listenerList.add(TreeWillExpandListener.class, l);
    }
    
    public void removeTreeWillExpandListener(TreeWillExpandListener l) {
        this.listenerList.remove(TreeWillExpandListener.class, l);
    }
    
    public TreeWillExpandListener[] getTreeWillExpandListeners() {
        return (TreeWillExpandListener[])this.listenerList.getListeners(TreeWillExpandListener.class);
    }
    
    public void fireTreeExpanded(TreePath treePath) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        if (this.uiTreeExpansionListener != null) {
            o = new TreeExpansionEvent(this, treePath);
            this.uiTreeExpansionListener.treeExpanded(o);
        }
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeExpansionListener.class && listenerList[i + 1] != this.uiTreeExpansionListener) {
                if (o == null) {
                    o = new TreeExpansionEvent(this, treePath);
                }
                ((TreeExpansionListener)listenerList[i + 1]).treeExpanded(o);
            }
        }
    }
    
    public void fireTreeCollapsed(TreePath treePath) {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        if (this.uiTreeExpansionListener != null) {
            o = new TreeExpansionEvent(this, treePath);
            this.uiTreeExpansionListener.treeCollapsed(o);
        }
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeExpansionListener.class && listenerList[i + 1] != this.uiTreeExpansionListener) {
                if (o == null) {
                    o = new TreeExpansionEvent(this, treePath);
                }
                ((TreeExpansionListener)listenerList[i + 1]).treeCollapsed(o);
            }
        }
    }
    
    public void fireTreeWillExpand(TreePath path) throws ExpandVetoException {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeWillExpandListener.class) {
                if (o == null) {
                    o = new TreeExpansionEvent(this, path);
                }
                ((TreeWillExpandListener)listenerList[i + 1]).treeWillExpand(o);
            }
        }
    }
    
    public void fireTreeWillCollapse(TreePath path) throws ExpandVetoException {
        Object[] listenerList = this.listenerList.getListenerList();
        Object o = null;
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeWillExpandListener.class) {
                if (o == null) {
                    o = new TreeExpansionEvent(this, path);
                }
                ((TreeWillExpandListener)listenerList[i + 1]).treeWillCollapse(o);
            }
        }
    }
    
    public void addTreeSelectionListener(TreeSelectionListener l) {
        this.listenerList.add(TreeSelectionListener.class, l);
        if (this.listenerList.getListenerCount(TreeSelectionListener.class) != 0 && this.selectionRedirector == null) {
            this.selectionRedirector = new JTree.TreeSelectionRedirector();
            this.selectionModel.addTreeSelectionListener(this.selectionRedirector);
        }
    }
    
    public void removeTreeSelectionListener(TreeSelectionListener l) {
        this.listenerList.remove(TreeSelectionListener.class, l);
        if (this.listenerList.getListenerCount(TreeSelectionListener.class) == 0 && this.selectionRedirector != null) {
            this.selectionModel.removeTreeSelectionListener(this.selectionRedirector);
            this.selectionRedirector = null;
        }
    }
    
    public TreeSelectionListener[] getTreeSelectionListeners() {
        return (TreeSelectionListener[])this.listenerList.getListeners(TreeSelectionListener.class);
    }
    
    protected void fireValueChanged(TreeSelectionEvent treeSelectionEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == TreeSelectionListener.class) {
                ((TreeSelectionListener)listenerList[i + 1]).valueChanged(treeSelectionEvent);
            }
        }
    }
    
    public void treeDidChange() {
        this.revalidate();
        this.repaint();
    }
    
    public void setVisibleRowCount(int visibleRowCount) {
        this.firePropertyChange("visibleRowCount", this.visibleRowCount, this.visibleRowCount = visibleRowCount);
        this.invalidate();
        if (this.accessibleContext != null) {
            ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
        }
    }
    
    public int getVisibleRowCount() {
        return this.visibleRowCount;
    }
    
    private void expandRoot() {
        TreeModel model = this.getModel();
        if (model != null && model.getRoot() != null) {
            this.expandPath(new TreePath(model.getRoot()));
        }
    }
    
    public TreePath getNextMatch(String upperCase, int i, Position.Bias bias) {
        int rowCount = this.getRowCount();
        if (upperCase == null) {
            throw new IllegalArgumentException();
        }
        if (i < 0 || i >= rowCount) {
            throw new IllegalArgumentException();
        }
        upperCase = upperCase.toUpperCase();
        int n = (bias == Position.Bias.Forward) ? 1 : -1;
        do {
            TreePath pathForRow = this.getPathForRow(i);
            if (this.convertValueToText(pathForRow.getLastPathComponent(), this.isRowSelected(i), this.isExpanded(i), true, i, false).toUpperCase().startsWith(upperCase)) {
                return pathForRow;
            }
            i = (i + n + rowCount) % rowCount;
        } while (i != i);
        return null;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector obj = new Vector();
        objectOutputStream.defaultWriteObject();
        if (this.cellRenderer != null && this.cellRenderer instanceof Serializable) {
            obj.addElement("cellRenderer");
            obj.addElement(this.cellRenderer);
        }
        if (this.cellEditor != null && this.cellEditor instanceof Serializable) {
            obj.addElement("cellEditor");
            obj.addElement(this.cellEditor);
        }
        if (this.treeModel != null && this.treeModel instanceof Serializable) {
            obj.addElement("treeModel");
            obj.addElement(this.treeModel);
        }
        if (this.selectionModel != null && this.selectionModel instanceof Serializable) {
            obj.addElement("selectionModel");
            obj.addElement(this.selectionModel);
        }
        Object archivableExpandedState = this.getArchivableExpandedState();
        if (archivableExpandedState != null) {
            obj.addElement("expandedState");
            obj.addElement(archivableExpandedState);
        }
        objectOutputStream.writeObject(obj);
        if (this.getUIClassID().equals("TreeUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.expandedState = new Hashtable();
        this.expandedStack = new Stack();
        Vector<Object> vector = (Vector)objectInputStream.readObject();
        int index = 0;
        int size = vector.size();
        if (index < size && vector.elementAt(index).equals("cellRenderer")) {
            this.cellRenderer = (TreeCellRenderer)vector.elementAt(++index);
            ++index;
        }
        if (index < size && vector.elementAt(index).equals("cellEditor")) {
            this.cellEditor = (TreeCellEditor)vector.elementAt(++index);
            ++index;
        }
        if (index < size && vector.elementAt(index).equals("treeModel")) {
            this.treeModel = (TreeModel)vector.elementAt(++index);
            ++index;
        }
        if (index < size && vector.elementAt(index).equals("selectionModel")) {
            this.selectionModel = (TreeSelectionModel)vector.elementAt(++index);
            ++index;
        }
        if (index < size && vector.elementAt(index).equals("expandedState")) {
            this.unarchiveExpandedState(vector.elementAt(++index));
            ++index;
        }
        if (this.listenerList.getListenerCount(TreeSelectionListener.class) != 0) {
            this.selectionRedirector = new JTree.TreeSelectionRedirector();
            this.selectionModel.addTreeSelectionListener(this.selectionRedirector);
        }
        if (this.treeModel != null) {
            this.treeModelListener = this.createTreeModelListener();
            if (this.treeModelListener != null) {
                this.treeModel.addTreeModelListener(this.treeModelListener);
            }
        }
    }
    
    private Object getArchivableExpandedState() {
        if (this.getModel() != null) {
            Enumeration<TreePath> keys = this.expandedState.keys();
            if (keys != null) {
                Vector vector = new Vector();
                while (keys.hasMoreElements()) {
                    TreePath key = (TreePath)keys.nextElement();
                    int[] modelIndexsForPath;
                    try {
                        modelIndexsForPath = this.getModelIndexsForPath(key);
                    }
                    catch (Error error) {
                        modelIndexsForPath = null;
                    }
                    if (modelIndexsForPath != null) {
                        vector.addElement(modelIndexsForPath);
                        vector.addElement(this.expandedState.get(key));
                    }
                    else {
                        continue;
                    }
                }
                return vector;
            }
        }
        return null;
    }
    
    private void unarchiveExpandedState(Object o) {
        if ((o instanceof Vector)) {
            Vector<Object> vector = (Vector)o;
            for (int i = vector.size() - 1; i >= 0; --i) {
                Boolean value = (Boolean)vector.elementAt(i--);
                try {
                    TreePath pathForIndexs = this.getPathForIndexs((int[])(int[])vector.elementAt(i));
                    if (pathForIndexs != null) {
                        this.expandedState.put(pathForIndexs, value);
                    }
                }
                catch (Error error) {}
            }
        }
    }
    
    private int[] getModelIndexsForPath(TreePath treePath) {
        if (treePath != null) {
            TreeModel model = this.getModel();
            int pathCount = treePath.getPathCount();
            int[] array = new int[pathCount - 1];
            Object o = model.getRoot();
            int i = 1;
            while (i < pathCount) {
                array[i - 1] = model.getIndexOfChild(o, treePath.getPathComponent(i));
                o = treePath.getPathComponent(i);
                if (array[i - 1] < 0) {
                    return null;
                }
                ++i;
            }
            return array;
        }
        return null;
    }
    
    private TreePath getPathForIndexs(int[] array) {
        if (array == null) {
            return null;
        }
        TreeModel model = this.getModel();
        if (model == null) {
            return null;
        }
        int length = array.length;
        Object o = model.getRoot();
        if (o == null) {
            return null;
        }
        TreePath pathByAddingChild = new TreePath(o);
        int i = 0;
        while (i < length) {
            o = model.getChild(o, array[i]);
            if (o == null) {
                return null;
            }
            pathByAddingChild = pathByAddingChild.pathByAddingChild(o);
            ++i;
        }
        return pathByAddingChild;
    }
    
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        int width = this.getPreferredSize().width;
        int visibleRowCount = this.getVisibleRowCount();
        int height = -1;
        if (this.isFixedRowHeight()) {
            height = visibleRowCount * this.getRowHeight();
        }
        else {
            TreeUI ui = this.getUI();
            if (ui != null && visibleRowCount > 0) {
                int rowCount = ui.getRowCount(this);
                if (rowCount >= visibleRowCount) {
                    Rectangle rowBounds2 = this.getRowBounds(visibleRowCount - 1);
                    if (rowBounds2 != null) {
                        height = rowBounds2.y + rowBounds2.height;
                    }
                }
                else if (rowCount > 0) {
                    Rectangle rowBounds = this.getRowBounds(0);
                    if (rowBounds != null) {
                        height = rowBounds.height * visibleRowCount;
                    }
                }
            }
            if (height == -1) {
                height = 16 * visibleRowCount;
            }
        }
        return new Dimension(width, height);
    }
    
    @Override
    public int getScrollableUnitIncrement(Rectangle rectangle, int n, int n2) {
        if (n == 1) {
            int closestRowForLocation = this.getClosestRowForLocation(0, rectangle.y);
            if (closestRowForLocation != -1) {
                Rectangle rowBounds = this.getRowBounds(closestRowForLocation);
                if (rowBounds.y != rectangle.y) {
                    if (n2 < 0) {
                        return Math.max(0, rectangle.y - rowBounds.y);
                    }
                    return rowBounds.y + rowBounds.height - rectangle.y;
                }
                else if (n2 < 0) {
                    if (closestRowForLocation != 0) {
                        return this.getRowBounds(closestRowForLocation - 1).height;
                    }
                }
                else {
                    return rowBounds.height;
                }
            }
            return 0;
        }
        else {
            return 4;
        }
    }
    
    @Override
    public int getScrollableBlockIncrement(Rectangle rectangle, int n, int n2) {
        return (n == 1) ? rectangle.height : rectangle.width;
    }
    
    @Override
    public boolean getScrollableTracksViewportWidth() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getWidth() > this.getPreferredSize().width;
    }
    
    @Override
    public boolean getScrollableTracksViewportHeight() {
        Container unwrappedParent = SwingUtilities.getUnwrappedParent(this);
        return unwrappedParent instanceof JViewport && unwrappedParent.getHeight() > this.getPreferredSize().height;
    }
    
    protected void setExpandedState(TreePath treePath, boolean b) {
        if (treePath != null) {
            TreePath item = treePath.getParentPath();
            Stack<TreePath> item2;
            if (this.expandedStack.size() == 0) {
                item2 = new Stack();
            }
            else {
                item2 = (Stack)this.expandedStack.pop();
            }
            try {
                while (item != null) {
                    if (this.isExpanded(item)) {
                        item = null;
                    }
                    else {
                        item2.push(item);
                        item = item.getParentPath();
                    }
                }
                for (int i = item2.size() - 1; i >= 0; --i) {
                    TreePath key = (TreePath)item2.pop();
                    if (!this.isExpanded(key)) {
                        try {
                            this.fireTreeWillExpand(key);
                        }
                        catch (ExpandVetoException ex) {
                            return;
                        }
                        this.expandedState.put(key, Boolean.TRUE);
                        this.fireTreeExpanded(key);
                        if (this.accessibleContext != null) {
                            ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
                        }
                    }
                }
            }
            finally {
                if (this.expandedStack.size() < JTree.TEMP_STACK_SIZE) {
                    item2.removeAllElements();
                    this.expandedStack.push(item2);
                }
            }
            if (!b) {
                Object value2 = this.expandedState.get(treePath);
                if (value2 != null && (boolean)value2) {
                    try {
                        this.fireTreeWillCollapse(treePath);
                    }
                    catch (ExpandVetoException ex3) {
                        return;
                    }
                    this.expandedState.put(treePath, Boolean.FALSE);
                    this.fireTreeCollapsed(treePath);
                    if (this.removeDescendantSelectedPaths(treePath, false) && !this.isPathSelected(treePath)) {
                        this.addSelectionPath(treePath);
                    }
                    if (this.accessibleContext != null) {
                        ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
                    }
                }
            }
            else {
                Object value = this.expandedState.get(treePath);
                if (value != null && (boolean)value) {
                    return;
                }
                try {
                    this.fireTreeWillExpand(treePath);
                }
                catch (ExpandVetoException ex2) {
                    return;
                }
                this.expandedState.put(treePath, Boolean.TRUE);
                this.fireTreeExpanded(treePath);
                if (this.accessibleContext != null) {
                    ((JTree.AccessibleJTree)this.accessibleContext).fireVisibleDataPropertyChange();
                }
            }
        }
    }
    
    protected Enumeration<TreePath> getDescendantToggledPaths(TreePath treePath) {
        if (treePath == null) {
            return null;
        }
        Vector<TreePath> vector = new Vector();
        Enumeration<TreePath> keys = this.expandedState.keys();
        while (keys.hasMoreElements()) {
            TreePath treePath2 = (TreePath)keys.nextElement();
            if (treePath.isDescendant(treePath2)) {
                vector.addElement(treePath2);
            }
            else {
                continue;
            }
        }
        return vector.elements();
    }
    
    protected void removeDescendantToggledPaths(Enumeration<TreePath> enumeration) {
        if (enumeration != null) {
            while (enumeration.hasMoreElements()) {
                Enumeration<TreePath> descendantToggledPaths = this.getDescendantToggledPaths((TreePath)enumeration.nextElement());
                if (descendantToggledPaths != null) {
                    while (descendantToggledPaths.hasMoreElements()) {
                        this.expandedState.remove(descendantToggledPaths.nextElement());
                    }
                }
                else {
                    continue;
                }
            }
        }
    }
    
    protected void clearToggledPaths() {
        this.expandedState.clear();
    }
    
    protected TreeModelListener createTreeModelListener() {
        return new JTree.TreeModelHandler();
    }
    
    protected boolean removeDescendantSelectedPaths(TreePath treePath, boolean b) {
        TreePath[] descendantSelectedPaths = this.getDescendantSelectedPaths(treePath, b);
        if (descendantSelectedPaths != null) {
            this.getSelectionModel().removeSelectionPaths(descendantSelectedPaths);
            return true;
        }
        return false;
    }
    
    private TreePath[] getDescendantSelectedPaths(TreePath treePath, boolean b) {
        TreeSelectionModel selectionModel = this.getSelectionModel();
        TreePath[] array = (selectionModel != null) ? selectionModel.getSelectionPaths() : null;
        if (array != null) {
            boolean n = false;
            for (int i = array.length - 1; i >= 0; --i) {
                if (array[i] != null && treePath.isDescendant(array[i]) && (!treePath.equals(array[i]) || b)) {
                    n = true;
                }
                else {
                    array[i] = null;
                }
            }
            if (n == 0) {
                array = null;
                return array;
            }
            return array;
        }
        else {
            return null;
        }
    }
    
    void removeDescendantSelectedPaths(TreeModelEvent event) {
        TreePath treePath = SwingUtilities2.getTreePath(event, this.getModel());
        Object[] children = event.getChildren();
        if (this.getSelectionModel() != null && treePath != null && children != null && children.length > 0) {
            for (int i = children.length - 1; i >= 0; --i) {
                this.removeDescendantSelectedPaths(treePath.pathByAddingChild(children[i]), true);
            }
        }
    }
    
    @Override
    void setUIProperty(String s, Object o) {
        if (s == "rowHeight") {
            if (!this.rowHeightSet) {
                this.setRowHeight(((Number)o).intValue());
                this.rowHeightSet = false;
            }
        }
        else if (s == "scrollsOnExpand") {
            if (!this.scrollsOnExpandSet) {
                this.setScrollsOnExpand((boolean)o);
                this.scrollsOnExpandSet = false;
            }
        }
        else if (s == "showsRootHandles") {
            if (!this.showsRootHandlesSet) {
                this.setShowsRootHandles((boolean)o);
                this.showsRootHandlesSet = false;
            }
        }
        else {
            super.setUIProperty(s, o);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",editable=" + (this.editable ? "true" : "false") + ",invokesStopCellEditing=" + (this.invokesStopCellEditing ? "true" : "false") + ",largeModel=" + (this.largeModel ? "true" : "false") + ",rootVisible=" + (this.rootVisible ? "true" : "false") + ",rowHeight=" + this.rowHeight + ",scrollsOnExpand=" + (this.scrollsOnExpand ? "true" : "false") + ",showsRootHandles=" + (this.showsRootHandles ? "true" : "false") + ",toggleClickCount=" + this.toggleClickCount + ",visibleRowCount=" + this.visibleRowCount;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTree.AccessibleJTree();
        }
        return this.accessibleContext;
    }
    
    static {
        JTree.TEMP_STACK_SIZE = 11;
    }
    
    public static final class DropLocation extends TransferHandler.DropLocation
    {
        private final TreePath path;
        private final int index;
        
        private DropLocation(Point point, TreePath path, int index) {
            super(point);
            this.path = path;
            this.index = index;
        }
        
        public int getChildIndex() {
            return this.index;
        }
        
        public TreePath getPath() {
            return this.path;
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[dropPoint=" + this.getDropPoint() + ",path=" + this.path + ",childIndex=" + this.index + "]";
        }
    }
    
    private class TreeTimer extends Timer
    {
        public TreeTimer() {
            super(2000, null);
            this.setRepeats(false);
        }
        
        public void fireActionPerformed(ActionEvent actionEvent) {
            JTree.this.expandRow(JTree.this.expandRow);
        }
    }
    
    protected static class EmptySelectionModel extends DefaultTreeSelectionModel
    {
        protected static final JTree.EmptySelectionModel sharedInstance;
        
        public static JTree.EmptySelectionModel sharedInstance() {
            return JTree.EmptySelectionModel.sharedInstance;
        }
        
        @Override
        public void setSelectionPaths(TreePath[] array) {
        }
        
        @Override
        public void addSelectionPaths(TreePath[] array) {
        }
        
        @Override
        public void removeSelectionPaths(TreePath[] array) {
        }
        
        @Override
        public void setSelectionMode(int n) {
        }
        
        @Override
        public void setRowMapper(RowMapper rowMapper) {
        }
        
        @Override
        public void addTreeSelectionListener(TreeSelectionListener treeSelectionListener) {
        }
        
        @Override
        public void removeTreeSelectionListener(TreeSelectionListener treeSelectionListener) {
        }
        
        @Override
        public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        @Override
        public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        }
        
        static {
            sharedInstance = new JTree.EmptySelectionModel();
        }
    }
    
    protected class TreeSelectionRedirector implements Serializable, TreeSelectionListener
    {
        @Override
        public void valueChanged(TreeSelectionEvent treeSelectionEvent) {
            JTree.this.fireValueChanged((TreeSelectionEvent)treeSelectionEvent.cloneWithSource(JTree.this));
        }
    }
    
    protected class TreeModelHandler implements TreeModelListener
    {
        @Override
        public void treeNodesChanged(TreeModelEvent treeModelEvent) {
        }
        
        @Override
        public void treeNodesInserted(TreeModelEvent treeModelEvent) {
        }
        
        @Override
        public void treeStructureChanged(TreeModelEvent event) {
            if (event == null) {
                return;
            }
            TreePath treePath = SwingUtilities2.getTreePath(event, JTree.this.getModel());
            if (treePath == null) {
                return;
            }
            if (treePath.getPathCount() == 1) {
                JTree.this.clearToggledPaths();
                Object root = JTree.this.treeModel.getRoot();
                if (root != null && !JTree.this.treeModel.isLeaf(root)) {
                    JTree.this.expandedState.put(treePath, Boolean.TRUE);
                }
            }
            else if (JTree.this.expandedState.get(treePath) != null) {
                Vector vector = new Vector(1);
                int expanded = JTree.this.isExpanded(treePath);
                vector.addElement(treePath);
                JTree.this.removeDescendantToggledPaths(vector.elements());
                if (!(expanded == 0)) {
                    TreeModel model = JTree.this.getModel();
                    if (model == null || model.isLeaf(treePath.getLastPathComponent())) {
                        JTree.this.collapsePath(treePath);
                    }
                    else {
                        JTree.this.expandedState.put(treePath, Boolean.TRUE);
                    }
                }
            }
            JTree.this.removeDescendantSelectedPaths(treePath, false);
            return;
        }
        
        @Override
        public void treeNodesRemoved(TreeModelEvent event) {
            if (event == null) {
                return;
            }
            TreePath treePath = SwingUtilities2.getTreePath(event, JTree.this.getModel());
            Object[] children = event.getChildren();
            if (children == null) {
                return;
            }
            Vector<TreePath> vector = new Vector(Math.max(1, children.length));
            for (int i = children.length - 1; i >= 0; --i) {
                TreePath pathByAddingChild = treePath.pathByAddingChild(children[i]);
                if (JTree.this.expandedState.get(pathByAddingChild) != null) {
                    vector.addElement(pathByAddingChild);
                }
            }
            if (vector.size() > 0) {
                JTree.this.removeDescendantToggledPaths(vector.elements());
            }
            TreeModel model = JTree.this.getModel();
            if (model == null || model.isLeaf(treePath.getLastPathComponent())) {
                JTree.this.expandedState.remove(treePath);
            }
            JTree.this.removeDescendantSelectedPaths(event);
            return;
        }
    }
    
    public static class DynamicUtilTreeNode extends DefaultMutableTreeNode
    {
        protected boolean hasChildren;
        protected Object childValue;
        protected boolean loadedChildren;
        
        public static void createChildren(DefaultMutableTreeNode defaultMutableTreeNode, Object o) {
            if ((o instanceof Vector)) {
                Vector<Object> vector = (Vector)o;
                for (int j = 0; j < vector.size(); ++j) {
                    defaultMutableTreeNode.add(new JTree.DynamicUtilTreeNode(vector.elementAt(j), vector.elementAt(j)));
                }
            }
            else if ((o instanceof Hashtable)) {
                Hashtable<Object, Object> hashtable = (Hashtable)o;
                Enumeration<Object> keys = hashtable.keys();
                while (keys.hasMoreElements()) {
                    Object nextElement = keys.nextElement();
                    defaultMutableTreeNode.add(new JTree.DynamicUtilTreeNode(nextElement, hashtable.get(nextElement)));
                }
            }
            else if ((o instanceof Object[])) {
                Object[] array = (Object[])(Object[])o;
                for (int i = 0; i < array.length; ++i) {
                    defaultMutableTreeNode.add(new JTree.DynamicUtilTreeNode(array[i], array[i]));
                }
            }
        }
        
        public DynamicUtilTreeNode(Object userObject, Object childValue) {
            super(userObject);
            this.loadedChildren = false;
            this.childValue = childValue;
            if (childValue != null) {
                if ((childValue instanceof Vector)) {
                    this.setAllowsChildren(true);
                }
                else if ((childValue instanceof Hashtable)) {
                    this.setAllowsChildren(true);
                }
                else if ((childValue instanceof Object[])) {
                    this.setAllowsChildren(true);
                }
                else {
                    this.setAllowsChildren(false);
                }
            }
            else {
                this.setAllowsChildren(false);
            }
        }
        
        @Override
        public boolean isLeaf() {
            return !this.getAllowsChildren();
        }
        
        @Override
        public int getChildCount() {
            if (!this.loadedChildren) {
                this.loadChildren();
            }
            return super.getChildCount();
        }
        
        protected void loadChildren() {
            this.loadedChildren = true;
            createChildren(this, this.childValue);
        }
        
        @Override
        public TreeNode getChildAt(int index) {
            if (!this.loadedChildren) {
                this.loadChildren();
            }
            return super.getChildAt(index);
        }
        
        @Override
        public Enumeration children() {
            if (!this.loadedChildren) {
                this.loadChildren();
            }
            return super.children();
        }
    }
    
    protected class AccessibleJTree extends JComponent.AccessibleJComponent implements AccessibleSelection, TreeSelectionListener, TreeModelListener, TreeExpansionListener
    {
        TreePath leadSelectionPath;
        Accessible leadSelectionAccessible;
        
        public AccessibleJTree() {
            TreeModel model = JTree.this.getModel();
            if (model != null) {
                model.addTreeModelListener(this);
            }
            JTree.this.addTreeExpansionListener(this);
            JTree.this.addTreeSelectionListener(this);
            this.leadSelectionPath = JTree.this.getLeadSelectionPath();
            this.leadSelectionAccessible = ((this.leadSelectionPath != null) ? new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, this.leadSelectionPath, JTree.this) : null);
        }
        
        @Override
        public void valueChanged(TreeSelectionEvent treeSelectionEvent) {
            this.firePropertyChange("AccessibleSelection", Boolean.valueOf(false), Boolean.valueOf(true));
        }
        
        public void fireVisibleDataPropertyChange() {
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
        }
        
        @Override
        public void treeNodesChanged(TreeModelEvent treeModelEvent) {
            this.fireVisibleDataPropertyChange();
        }
        
        @Override
        public void treeNodesInserted(TreeModelEvent treeModelEvent) {
            this.fireVisibleDataPropertyChange();
        }
        
        @Override
        public void treeNodesRemoved(TreeModelEvent treeModelEvent) {
            this.fireVisibleDataPropertyChange();
        }
        
        @Override
        public void treeStructureChanged(TreeModelEvent treeModelEvent) {
            this.fireVisibleDataPropertyChange();
        }
        
        @Override
        public void treeCollapsed(TreeExpansionEvent treeExpansionEvent) {
            this.fireVisibleDataPropertyChange();
            TreePath path = treeExpansionEvent.getPath();
            if (path != null) {
                this.firePropertyChange("AccessibleState", null, new PropertyChangeEvent(new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, path, null), "AccessibleState", AccessibleState.EXPANDED, AccessibleState.COLLAPSED));
            }
        }
        
        @Override
        public void treeExpanded(TreeExpansionEvent treeExpansionEvent) {
            this.fireVisibleDataPropertyChange();
            TreePath path = treeExpansionEvent.getPath();
            if (path != null) {
                this.firePropertyChange("AccessibleState", null, new PropertyChangeEvent(new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, path, null), "AccessibleState", AccessibleState.COLLAPSED, AccessibleState.EXPANDED));
            }
        }
        
        void fireActiveDescendantPropertyChange(TreePath treePath, TreePath treePath2) {
            if (treePath != treePath2) {
                this.firePropertyChange("AccessibleActiveDescendant", (treePath != null) ? new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, treePath, null) : null, (treePath2 != null) ? new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, treePath2, null) : null);
            }
        }
        
        private AccessibleContext getCurrentAccessibleContext() {
            Component currentComponent = this.getCurrentComponent();
            if ((currentComponent instanceof Accessible)) {
                return currentComponent.getAccessibleContext();
            }
            return null;
        }
        
        private Component getCurrentComponent() {
            TreeModel model = JTree.this.getModel();
            if (model == null) {
                return null;
            }
            Object root = model.getRoot();
            if (root == null) {
                return null;
            }
            TreePath treePath = new TreePath(root);
            if (JTree.this.isVisible(treePath)) {
                TreeCellRenderer cellRenderer = JTree.this.getCellRenderer();
                TreeUI ui = JTree.this.getUI();
                if (ui != null) {
                    int rowForPath = ui.getRowForPath(JTree.this, treePath);
                    int leadSelectionRow = JTree.this.getLeadSelectionRow();
                    return cellRenderer.getTreeCellRendererComponent(JTree.this, root, JTree.this.isPathSelected(treePath), JTree.this.isExpanded(treePath), model.isLeaf(root), rowForPath, JTree.this.isFocusOwner() && leadSelectionRow == rowForPath);
                }
            }
            return null;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TREE;
        }
        
        @Override
        public Accessible getAccessibleAt(Point point) {
            TreePath closestPathForLocation = JTree.this.getClosestPathForLocation(point.x, point.y);
            if (closestPathForLocation != null) {
                return new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, closestPathForLocation, null);
            }
            return null;
        }
        
        @Override
        public int getAccessibleChildrenCount() {
            TreeModel model = JTree.this.getModel();
            if (model == null) {
                return 0;
            }
            if (JTree.this.isRootVisible()) {
                return 1;
            }
            Object root = model.getRoot();
            if (root == null) {
                return 0;
            }
            return model.getChildCount(root);
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            TreeModel model = JTree.this.getModel();
            if (model == null) {
                return null;
            }
            Object root = model.getRoot();
            if (root == null) {
                return null;
            }
            if (JTree.this.isRootVisible()) {
                if (n == 0) {
                    Object[] path = { root };
                    if (path[0] == null) {
                        return null;
                    }
                    return new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, new TreePath(path), JTree.this);
                }
                else {
                    return null;
                }
            }
            else {
                int childCount = model.getChildCount(root);
                if (n < 0 || n >= childCount) {
                    return null;
                }
                Object child = model.getChild(root, n);
                if (child == null) {
                    return null;
                }
                return new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, new TreePath(new Object[] { root, child }), JTree.this);
            }
        }
        
        @Override
        public int getAccessibleIndexInParent() {
            return super.getAccessibleIndexInParent();
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            Object[] path = { JTree.this.treeModel.getRoot() };
            if (path[0] == null) {
                return 0;
            }
            if (JTree.this.isPathSelected(new TreePath(path))) {
                return 1;
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            if (n == 0) {
                Object[] path = { JTree.this.treeModel.getRoot() };
                if (path[0] == null) {
                    return null;
                }
                TreePath treePath = new TreePath(path);
                if (JTree.this.isPathSelected(treePath)) {
                    return new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, treePath, JTree.this);
                }
            }
            return null;
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            if (n == 0) {
                Object[] path = { JTree.this.treeModel.getRoot() };
                return path[0] != null && JTree.this.isPathSelected(new TreePath(path));
            }
            return false;
        }
        
        @Override
        public void addAccessibleSelection(int n) {
            TreeModel model = JTree.this.getModel();
            if (model != null && n == 0) {
                Object[] path = { model.getRoot() };
                if (path[0] == null) {
                    return;
                }
                JTree.this.addSelectionPath(new TreePath(path));
            }
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            TreeModel model = JTree.this.getModel();
            if (model != null && n == 0) {
                Object[] path = { model.getRoot() };
                if (path[0] == null) {
                    return;
                }
                JTree.this.removeSelectionPath(new TreePath(path));
            }
        }
        
        @Override
        public void clearAccessibleSelection() {
            for (int accessibleChildrenCount = this.getAccessibleChildrenCount(), i = 0; i < accessibleChildrenCount; ++i) {
                this.removeAccessibleSelection(i);
            }
        }
        
        @Override
        public void selectAllAccessibleSelection() {
            TreeModel model = JTree.this.getModel();
            if (model != null) {
                Object[] path = { model.getRoot() };
                if (path[0] == null) {
                    return;
                }
                JTree.this.addSelectionPath(new TreePath(path));
            }
        }
        
        protected class AccessibleJTreeNode extends AccessibleContext implements Accessible, AccessibleComponent, AccessibleSelection, AccessibleAction
        {
            private JTree tree;
            private TreeModel treeModel;
            private Object obj;
            private TreePath path;
            private Accessible accessibleParent;
            private int index;
            private boolean isLeaf;
            
            public AccessibleJTreeNode(JTree tree, TreePath path, Accessible accessibleParent) {
                this.tree = null;
                this.treeModel = null;
                this.obj = null;
                this.path = null;
                this.accessibleParent = null;
                this.index = 0;
                this.isLeaf = false;
                this.tree = tree;
                this.path = path;
                this.accessibleParent = accessibleParent;
                this.treeModel = tree.getModel();
                this.obj = path.getLastPathComponent();
                if (this.treeModel != null) {
                    this.isLeaf = this.treeModel.isLeaf(this.obj);
                }
            }
            
            private TreePath getChildTreePath(int n) {
                if (n < 0 || n >= this.getAccessibleChildrenCount()) {
                    return null;
                }
                Object child = this.treeModel.getChild(this.obj, n);
                Object[] path = this.path.getPath();
                Object[] path2 = new Object[path.length + 1];
                System.arraycopy(path, 0, path2, 0, path.length);
                path2[path2.length - 1] = child;
                return new TreePath(path2);
            }
            
            @Override
            public AccessibleContext getAccessibleContext() {
                return this;
            }
            
            private AccessibleContext getCurrentAccessibleContext() {
                Component currentComponent = this.getCurrentComponent();
                if ((currentComponent instanceof Accessible)) {
                    return currentComponent.getAccessibleContext();
                }
                return null;
            }
            
            private Component getCurrentComponent() {
                if (this.tree.isVisible(this.path)) {
                    TreeCellRenderer cellRenderer = this.tree.getCellRenderer();
                    if (cellRenderer == null) {
                        return null;
                    }
                    TreeUI ui = this.tree.getUI();
                    if (ui != null) {
                        return cellRenderer.getTreeCellRendererComponent(this.tree, this.obj, this.tree.isPathSelected(this.path), this.tree.isExpanded(this.path), this.isLeaf, ui.getRowForPath(JTree.this, this.path), false);
                    }
                }
                return null;
            }
            
            @Override
            public String getAccessibleName() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    String accessibleName = currentAccessibleContext.getAccessibleName();
                    if (accessibleName != null && accessibleName != "") {
                        return currentAccessibleContext.getAccessibleName();
                    }
                    return null;
                }
                else {
                    if (this.accessibleName != null && this.accessibleName != "") {
                        return this.accessibleName;
                    }
                    return (String)JTree.this.getClientProperty("AccessibleName");
                }
            }
            
            @Override
            public void setAccessibleName(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleName(s);
                }
                else {
                    super.setAccessibleName(s);
                }
            }
            
            @Override
            public String getAccessibleDescription() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleDescription();
                }
                return super.getAccessibleDescription();
            }
            
            @Override
            public void setAccessibleDescription(String s) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.setAccessibleDescription(s);
                }
                else {
                    super.setAccessibleDescription(s);
                }
            }
            
            @Override
            public AccessibleRole getAccessibleRole() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getAccessibleRole();
                }
                return AccessibleRole.UNKNOWN;
            }
            
            @Override
            public AccessibleStateSet getAccessibleStateSet() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                AccessibleStateSet accessibleStateSet;
                if (currentAccessibleContext != null) {
                    accessibleStateSet = currentAccessibleContext.getAccessibleStateSet();
                }
                else {
                    accessibleStateSet = new AccessibleStateSet();
                }
                if (this.isShowing()) {
                    accessibleStateSet.add(AccessibleState.SHOWING);
                }
                else if (accessibleStateSet.contains(AccessibleState.SHOWING)) {
                    accessibleStateSet.remove(AccessibleState.SHOWING);
                }
                if (this.isVisible()) {
                    accessibleStateSet.add(AccessibleState.VISIBLE);
                }
                else if (accessibleStateSet.contains(AccessibleState.VISIBLE)) {
                    accessibleStateSet.remove(AccessibleState.VISIBLE);
                }
                if (this.tree.isPathSelected(this.path)) {
                    accessibleStateSet.add(AccessibleState.SELECTED);
                }
                if (this.path == JTree.this.getLeadSelectionPath()) {
                    accessibleStateSet.add(AccessibleState.ACTIVE);
                }
                if (!this.isLeaf) {
                    accessibleStateSet.add(AccessibleState.EXPANDABLE);
                }
                if (this.tree.isExpanded(this.path)) {
                    accessibleStateSet.add(AccessibleState.EXPANDED);
                }
                else {
                    accessibleStateSet.add(AccessibleState.COLLAPSED);
                }
                if (this.tree.isEditable()) {
                    accessibleStateSet.add(AccessibleState.EDITABLE);
                    return accessibleStateSet;
                }
                return accessibleStateSet;
            }
            
            @Override
            public Accessible getAccessibleParent() {
                if (this.accessibleParent == null) {
                    Object[] path = this.path.getPath();
                    if (path.length > 1) {
                        Object o = path[path.length - 2];
                        if (this.treeModel != null) {
                            this.index = this.treeModel.getIndexOfChild(o, this.obj);
                        }
                        Object[] path2 = new Object[path.length - 1];
                        System.arraycopy(path, 0, path2, 0, path.length - 1);
                        this.setAccessibleParent(this.accessibleParent = new JTree.AccessibleJTree.AccessibleJTreeNode(this.tree, new TreePath(path2), null));
                    }
                    else if (this.treeModel != null) {
                        this.accessibleParent = this.tree;
                        this.index = 0;
                        this.setAccessibleParent(this.accessibleParent);
                    }
                }
                return this.accessibleParent;
            }
            
            @Override
            public int getAccessibleIndexInParent() {
                if (this.accessibleParent == null) {
                    this.getAccessibleParent();
                }
                Object[] path = this.path.getPath();
                if (path.length > 1) {
                    Object o = path[path.length - 2];
                    if (this.treeModel != null) {
                        this.index = this.treeModel.getIndexOfChild(o, this.obj);
                    }
                }
                return this.index;
            }
            
            @Override
            public int getAccessibleChildrenCount() {
                return this.treeModel.getChildCount(this.obj);
            }
            
            @Override
            public Accessible getAccessibleChild(int n) {
                if (n < 0 || n >= this.getAccessibleChildrenCount()) {
                    return null;
                }
                Object child = this.treeModel.getChild(this.obj, n);
                Object[] path = this.path.getPath();
                Object[] path2 = new Object[path.length + 1];
                System.arraycopy(path, 0, path2, 0, path.length);
                path2[path2.length - 1] = child;
                return new JTree.AccessibleJTree.AccessibleJTreeNode(JTree.this, new TreePath(path2), this);
            }
            
            @Override
            public Locale getLocale() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    return currentAccessibleContext.getLocale();
                }
                return this.tree.getLocale();
            }
            
            @Override
            public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.addPropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.addPropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    currentAccessibleContext.removePropertyChangeListener(propertyChangeListener);
                }
                else {
                    super.removePropertyChangeListener(propertyChangeListener);
                }
            }
            
            @Override
            public AccessibleAction getAccessibleAction() {
                return this;
            }
            
            @Override
            public AccessibleComponent getAccessibleComponent() {
                return this;
            }
            
            @Override
            public AccessibleSelection getAccessibleSelection() {
                if (this.getCurrentAccessibleContext() != null && this.isLeaf) {
                    return this.getCurrentAccessibleContext().getAccessibleSelection();
                }
                return this;
            }
            
            @Override
            public AccessibleText getAccessibleText() {
                if (this.getCurrentAccessibleContext() != null) {
                    return this.getCurrentAccessibleContext().getAccessibleText();
                }
                return null;
            }
            
            @Override
            public AccessibleValue getAccessibleValue() {
                if (this.getCurrentAccessibleContext() != null) {
                    return this.getCurrentAccessibleContext().getAccessibleValue();
                }
                return null;
            }
            
            @Override
            public Color getBackground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBackground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBackground();
                }
                return null;
            }
            
            @Override
            public void setBackground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBackground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBackground(color);
                    }
                }
            }
            
            @Override
            public Color getForeground() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getForeground();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getForeground();
                }
                return null;
            }
            
            @Override
            public void setForeground(Color color) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setForeground(color);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setForeground(color);
                    }
                }
            }
            
            @Override
            public Cursor getCursor() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getCursor();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getCursor();
                }
                Accessible accessibleParent = this.getAccessibleParent();
                if ((accessibleParent instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)accessibleParent).getCursor();
                }
                return null;
            }
            
            @Override
            public void setCursor(Cursor cursor) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setCursor(cursor);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setCursor(cursor);
                    }
                }
            }
            
            @Override
            public Font getFont() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFont();
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFont();
                }
                return null;
            }
            
            @Override
            public void setFont(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setFont(font);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setFont(font);
                    }
                }
            }
            
            @Override
            public FontMetrics getFontMetrics(Font font) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getFontMetrics(font);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getFontMetrics(font);
                }
                return null;
            }
            
            @Override
            public boolean isEnabled() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isEnabled();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isEnabled();
            }
            
            @Override
            public void setEnabled(boolean b) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setEnabled(b);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setEnabled(b);
                    }
                }
            }
            
            @Override
            public boolean isVisible() {
                Rectangle pathBounds = this.tree.getPathBounds(this.path);
                Rectangle visibleRect = this.tree.getVisibleRect();
                return pathBounds != null && visibleRect != null && visibleRect.intersects(pathBounds);
            }
            
            @Override
            public void setVisible(boolean b) {
            }
            
            @Override
            public boolean isShowing() {
                return this.tree.isShowing() && this.isVisible();
            }
            
            @Override
            public boolean contains(Point p) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getBounds().contains(p);
                }
                Component currentComponent = this.getCurrentComponent();
                if (currentComponent != null) {
                    return currentComponent.getBounds().contains(p);
                }
                return this.getBounds().contains(p);
            }
            
            @Override
            public Point getLocationOnScreen() {
                if (this.tree != null) {
                    Point locationOnScreen = this.tree.getLocationOnScreen();
                    Rectangle pathBounds = this.tree.getPathBounds(this.path);
                    if (locationOnScreen != null && pathBounds != null) {
                        Point point = new Point(pathBounds.x, pathBounds.y);
                        point.translate(locationOnScreen.x, locationOnScreen.y);
                        return point;
                    }
                    return null;
                }
                else {
                    return null;
                }
            }
            
            protected Point getLocationInJTree() {
                Rectangle pathBounds = this.tree.getPathBounds(this.path);
                if (pathBounds != null) {
                    return pathBounds.getLocation();
                }
                return null;
            }
            
            @Override
            public Point getLocation() {
                Rectangle bounds = this.getBounds();
                if (bounds != null) {
                    return bounds.getLocation();
                }
                return null;
            }
            
            @Override
            public void setLocation(Point point) {
            }
            
            @Override
            public Rectangle getBounds() {
                Rectangle pathBounds = this.tree.getPathBounds(this.path);
                Accessible accessibleParent = this.getAccessibleParent();
                if (accessibleParent != null && accessibleParent instanceof JTree.AccessibleJTree.AccessibleJTreeNode) {
                    Point locationInJTree = ((JTree.AccessibleJTree.AccessibleJTreeNode)accessibleParent).getLocationInJTree();
                    if (locationInJTree != null && pathBounds != null) {
                        pathBounds.translate(-locationInJTree.x, -locationInJTree.y);
                    }
                    else {
                        return null;
                    }
                }
                return pathBounds;
            }
            
            @Override
            public void setBounds(Rectangle rectangle) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setBounds(rectangle);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setBounds(rectangle);
                    }
                }
            }
            
            @Override
            public Dimension getSize() {
                return this.getBounds().getSize();
            }
            
            @Override
            public void setSize(Dimension dimension) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).setSize(dimension);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.setSize(dimension);
                    }
                }
            }
            
            @Override
            public Accessible getAccessibleAt(Point point) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).getAccessibleAt(point);
                }
                return null;
            }
            
            @Override
            public boolean isFocusTraversable() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    return ((AccessibleComponent)currentAccessibleContext).isFocusTraversable();
                }
                Component currentComponent = this.getCurrentComponent();
                return currentComponent != null && currentComponent.isFocusTraversable();
            }
            
            @Override
            public void requestFocus() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).requestFocus();
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.requestFocus();
                    }
                }
            }
            
            @Override
            public void addFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).addFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.addFocusListener(l);
                    }
                }
            }
            
            @Override
            public void removeFocusListener(FocusListener l) {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if ((currentAccessibleContext instanceof AccessibleComponent)) {
                    ((AccessibleComponent)currentAccessibleContext).removeFocusListener(l);
                }
                else {
                    Component currentComponent = this.getCurrentComponent();
                    if (currentComponent != null) {
                        currentComponent.removeFocusListener(l);
                    }
                }
            }
            
            @Override
            public int getAccessibleSelectionCount() {
                int n = 0;
                for (int accessibleChildrenCount = this.getAccessibleChildrenCount(), i = 0; i < accessibleChildrenCount; ++i) {
                    if (this.tree.isPathSelected(this.getChildTreePath(i))) {
                        ++n;
                    }
                }
                return n;
            }
            
            @Override
            public Accessible getAccessibleSelection(int n) {
                int accessibleChildrenCount = this.getAccessibleChildrenCount();
                if (n < 0 || n >= accessibleChildrenCount) {
                    return null;
                }
                for (int n2 = 0, n3 = 0; n3 < accessibleChildrenCount && n >= n2; ++n3) {
                    TreePath childTreePath = this.getChildTreePath(n3);
                    if (this.tree.isPathSelected(childTreePath)) {
                        if (n2 == n) {
                            return new JTree.AccessibleJTree.AccessibleJTreeNode(this.tree, childTreePath, this);
                        }
                        ++n2;
                    }
                }
                return null;
            }
            
            @Override
            public boolean isAccessibleChildSelected(int n) {
                int accessibleChildrenCount = this.getAccessibleChildrenCount();
                return n >= 0 && n < accessibleChildrenCount && this.tree.isPathSelected(this.getChildTreePath(n));
            }
            
            @Override
            public void addAccessibleSelection(int n) {
                if (JTree.this.getModel() != null && n >= 0 && n < this.getAccessibleChildrenCount()) {
                    JTree.this.addSelectionPath(this.getChildTreePath(n));
                }
            }
            
            @Override
            public void removeAccessibleSelection(int n) {
                if (JTree.this.getModel() != null && n >= 0 && n < this.getAccessibleChildrenCount()) {
                    JTree.this.removeSelectionPath(this.getChildTreePath(n));
                }
            }
            
            @Override
            public void clearAccessibleSelection() {
                for (int accessibleChildrenCount = this.getAccessibleChildrenCount(), i = 0; i < accessibleChildrenCount; ++i) {
                    this.removeAccessibleSelection(i);
                }
            }
            
            @Override
            public void selectAllAccessibleSelection() {
                if (JTree.this.getModel() != null) {
                    for (int accessibleChildrenCount = this.getAccessibleChildrenCount(), i = 0; i < accessibleChildrenCount; ++i) {
                        JTree.this.addSelectionPath(this.getChildTreePath(i));
                    }
                }
            }
            
            @Override
            public int getAccessibleActionCount() {
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (currentAccessibleContext != null) {
                    AccessibleAction accessibleAction = currentAccessibleContext.getAccessibleAction();
                    if (accessibleAction != null) {
                        return accessibleAction.getAccessibleActionCount() + (this.isLeaf ? 0 : 1);
                    }
                }
                return !this.isLeaf;
            }
            
            @Override
            public String getAccessibleActionDescription(int n) {
                if (n < 0 || n >= this.getAccessibleActionCount()) {
                    return null;
                }
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (n == 0) {
                    return AccessibleAction.TOGGLE_EXPAND;
                }
                if (currentAccessibleContext != null) {
                    AccessibleAction accessibleAction = currentAccessibleContext.getAccessibleAction();
                    if (accessibleAction != null) {
                        return accessibleAction.getAccessibleActionDescription(n - 1);
                    }
                }
                return null;
            }
            
            @Override
            public boolean doAccessibleAction(int n) {
                if (n < 0 || n >= this.getAccessibleActionCount()) {
                    return false;
                }
                AccessibleContext currentAccessibleContext = this.getCurrentAccessibleContext();
                if (n == 0) {
                    if (JTree.this.isExpanded(this.path)) {
                        JTree.this.collapsePath(this.path);
                    }
                    else {
                        JTree.this.expandPath(this.path);
                    }
                    return true;
                }
                else {
                    if (currentAccessibleContext != null) {
                        AccessibleAction accessibleAction = currentAccessibleContext.getAccessibleAction();
                        if (accessibleAction != null) {
                            return accessibleAction.doAccessibleAction(n - 1);
                        }
                    }
                    return false;
                }
            }
        }
    }
}
